
/*
 *  Fri Mar 31 03:48:40 EEST 2000
 *  Mon Apr 10 23:09:21 EEST 2000
 *    Added some commentary.
 *  Mon May  1 16:48:48 EEST 2000
 *    Loops have changed, 110 T-times now. Dont believe commentary.
 *
 *  Print DTMF tables for R58 dtmf generation with the i8254
 *
 *  See dtmfbang.asm.
 *  Each sample-byte is pre-scaled,
 *  Each sample-byte is followed by the address LSB of next sample.
 *
 *  Note a table need not emit just a single cycle, if it still fits in
 *  256 bytes it might emit 2 or 3 cycles to get better accuracy.
 *
 *  This is magic-number-ridden. The 40320.0 comes from
 *  timer input clock, 4.032 MHz divided by PWM-cell-time, 110 cycles.
 *  Constants 24.0 and 25.0 scale the sum of tones from minimum to
 *  almost maximum.
 */

#include <stdio.h>
#include <stdlib.h>
#include <math.h>

static int dot;

#define ZZZ (4032000.0 / 110)

static void
tab(int norm)
{
	int N, M;
	int i, j;
	double result, err;
	double x, d;

	M = 0;                    /* number of cycles in table */
more:
	M++;
	x = ZZZ / norm;           /* number of cycle divisions for norm */
	N = (M * x) + 0.5;        /* total divisions must have continuity */
	x = (double) N / M;       /* result after fitting to M cycles */

	result = ZZZ / x;
	err = 100.0 * (result - norm) / norm;

	printf("! %4d is %6.1f, %3d samples, %d cycles, %3.1f%% error",
		norm, result, N, M, err);

	if (fabs(err) > 0.4) {
		printf(", retry...\n");
		goto more;
	}
	printf(", ok.\n");

	/*
	 *  Each table stays inside a 0xNN00 page
	 */
	if (2 * N > 0x100) {
		fprintf(stderr, "oops, table grows to %d bytes. Sorry.\n", 2 * N);
		exit(1);
	}
	if (dot + 2 * N > 0x100) {
		printf(".align 8, 0xFF ! slack %d\n", 0x100 - dot);
		dot = 0;
	}
	printf("1:\ndtmf_cycle_%d_Hz:\n", norm);

	j = 0;
	for (i = 0; i < N; ++i) {
		/*
		 *  amplitude +/- 27
		 *  DC at 27
		 */
		if (j++ == 0)
			printf("\t.byte");

		d = sin((double) i * 2 * M_PI / x) * 27.0 + 27.0;

		printf(" %d,%d", (int) (d + 0.5), dot + ((i + 1) % N) * 2);

		if (j == 8) {
			j = 0;
			printf("\n");
		} else {
			if (i != N - 1)
				printf(",");
		}
	}
		d = sin((double) i * 2 * M_PI / x) * 27.0 + 27.0;
		printf(" ! nxt %d ", (int) (d + 0.5));

	printf("\n");
	printf("\tASSERT((. - 1b) == %d); ASSERT(HI(. - 1) == HI(1b))\n", 2 * N);
	printf("\n");

	dot += 2 * N;
}

main()
{
	/*
	 *  Save a byte or two by grouping wrt size/alignment.
	 */
	printf(".align 8, 0xFF\n");
	dot = 0;

	tab(1633);
	tab(1477);
	tab( 770);
	tab(1336);
	tab( 852);
	tab(1209);
	tab( 941);
	tab( 697);

	tab(1750); /* testing purposes */

	exit(0);
}

